package com.muyu.etl.rule.engine.core.classLoad;

import com.alibaba.fastjson2.JSON;
import com.muyu.etl.rule.engine.bean.constant.FileConstants;

import com.muyu.etl.rule.engine.util.FileLoadUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * @Author WangXin
 * @Data 2024/8/22
 * @Description java 字节码文件 加载器
 * @Version 1.0.0
 */
public class JavaBinaryClassLoader extends ClassLoader {

    private static final Logger log = LogManager.getLogger(JavaBinaryClassLoader.class.getName());

    private static final JavaBinaryClassLoader loader = new JavaBinaryClassLoader();

    /**
     * 自定义路径获取类的class对象
     *
     * @param name         类名
     * @param pack         包名
     * @param locationFile 文件地址
     * @return class对象
     */
    private static Class<?> loadClassByNameAndLocation(String name, String pack, File locationFile) {
        //将class文件的数据读入到byte数组中
        byte[] datas = loader.loadClassData(name, locationFile);
        //通过byte数组加载Class对象
        Class<?> aClass = loader.defineClass(pack + name, datas, 0, datas.length);
        log.info("成功加载规则引擎 -- {}", aClass.getName());
        return aClass;
    }


    /**
     * @param pack     类所在的包名 com.muyu.bean. 后面需要带个点
     * @param location 包所在路径
     * @return
     */
    public static Map<String, Class<?>> loadClassByLocation(String pack, String location) {
        //列出所有的class文件
        List<File> classFiles = FileLoadUtil.getClassFiles(location + FileConstants.FILE_DELIMITER + pack.replaceAll("\\.", FileConstants.FILE_DELIMITER));
        log.info("class文件---->{}", JSON.toJSONString(classFiles));
        HashMap<String, Class<?>> map = new HashMap<>();
        classFiles.forEach(file -> {
            String name = file.getName().substring(0, file.getName().lastIndexOf(FileConstants.CLASS_SUFFIX));
//            Class<?> c = loadClassByNameAndLocation(name, pack + ".", new File(location+ FileConstants.FILE_DELIMITER +pack.replaceAll("\\.",FileConstants.FILE_DELIMITER)));
            try {
                log.info("转class对象请求入参---》name:{},pack:{}，path:{}", name, pack, file.getAbsolutePath());
                Class<?> value = urlLoadClassByNameAndLocation(name, pack, file.getAbsolutePath());
                map.put(name, value);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }

        });
        return map;
    }

    /**
     * 将文件转换成字节数组
     *
     * @param name         文件名称
     * @param locationFile 文件
     * @return 字节数组
     */
    public byte[] loadClassData(String name, File locationFile) {
        byte[] byteArray = null;
        try {
            FileInputStream fis = new FileInputStream(locationFile + FileConstants.FILE_DELIMITER + name + FileConstants.CLASS_SUFFIX);
            // 创建ByteArrayOutputStream对象
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            // 创建一个缓冲区
            byte[] buffer = new byte[1024];
            int length;
            // 读取文件内容并写入缓冲区
            while ((length = fis.read(buffer)) != -1) {
                bos.write(buffer, 0, length);
            }
            // 将缓冲区中的内容转换为byte数组
            byteArray = bos.toByteArray();
        } catch (IOException e) {
            log.error("[【{}】文件转换成字节数组错误]：{}", name, e.getMessage(), e);
            throw new RuntimeException(e);
        }
        return byteArray;
    }

    /**
     * 自定义路径获取类的class对象
     *
     * @param name 类名
     * @param pack 包名
     * @param path 文件地址
     * @return class对象
     */
    private static Class<?> urlLoadClassByNameAndLocation(String name, String pack, String path) throws IOException {
        Path classFilePath = Paths.get(path);
        URL[] urls = new URL[]{new File(path).toURI().toURL()};
        ExternalClassLoader externalClassLoader = new ExternalClassLoader(urls);
        // 注意类名必须是完全限定名（包括包名）
        log.info("[externalClassLoader]请求入参---》classPath:{},className:{}", classFilePath, pack + "." + name);
        Class<?> clazz = externalClassLoader.loadClassFromPath(classFilePath, pack + "." + name);
        return clazz;
    }

    public static void main(String[] args) {
        File file = new File("D:\\idea_data\\ETL\\RuleEngine\\src\\main\\java\\com\\muyu\\engine\\core\\classLoad\\ExternalClassLoader.java");
        String path = file.getPath();
        String absolutePath = file.getAbsolutePath();
        System.out.println(path);
        System.out.println(absolutePath.substring(0, absolutePath.lastIndexOf(FileConstants.CLASS_SUFFIX)));
    }


}
