package com.muyu.etl.rule.engine.util;

import com.alibaba.fastjson2.JSONObject;
import com.muyu.etl.rule.engine.bean.constant.FileConstants;
import com.muyu.etl.rule.engine.core.complier.SourceCodeCompiler;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.*;
import java.util.Arrays;
import java.util.List;

/**
 * @Author WangXin
 * @Data 2024/8/22
 * @Description 文件扫描
 * @Version 1.0.0
 */

public class FileLoadUtil {

    private static final Logger log = LogManager.getLogger(SourceCodeCompiler.class.getName());

    /**
     * 判断文件是否存在
     *
     * @param file 文件
     * @return 存在返回文件不存在报错
     */
    public static File isFilePresence(File file) {
        if (file.exists()) {
            return file;
        }
        log.error("[文件路径错误]");
        throw new RuntimeException("[文件路径错误]");
    }

    public static File[] isFilesExist(File[] files) {
        if (files == null || files.length == 0) {
            log.error("该文件夹下没有文件,请创建文件");
            throw new RuntimeException("该文件夹下没有文件,请创建文件");
        }
        return files;
    }

    /**
     * 通过文件路径获取该文件夹下的所有文件
     *
     * @param folderPath 文件夹路径
     * @return java文件
     */
    public static List<File> getJavaFiles(String folderPath) {
        File folder = new File(folderPath);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        File[] files = folder.listFiles();
        return Arrays.stream(isFilesExist(files)).filter(FileLoadUtil::isFileType).toList();
    }

    /**
     * 通过文件路径获取该文件夹下的所有文件
     *
     * @param folderPath 文件夹路径
     * @return class文件
     */
    public static List<File> getClassFiles(String folderPath) {
        File folder = new File(folderPath);
        if (!folder.exists()) {
            folder.mkdirs();
        }
        File[] files = folder.listFiles();
        List<File> list = Arrays.stream(isFilesExist(files)).filter(FileLoadUtil::isClassFileType).toList();
        log.info("[{}]路劲下的class文件有[{}]", folderPath, JSONObject.toJSONString(list));
        return list;
    }

    /**
     * 判断java文件类型
     *
     * @param file 文件
     * @return 布尔类型的值
     */
    public static Boolean isFileType(File file) {
        return file.getName().endsWith(FileConstants.JAVA_SUFFIX);
    }

    /**
     * 判断class文件类型
     *
     * @param file 文件
     * @return 布尔类型的值
     */
    public static Boolean isClassFileType(File file) {
        return file.getName().endsWith(FileConstants.CLASS_SUFFIX);
    }

    /**
     * 创建java文件
     */
    public static File createJavaFile(String filePath, String fileName) {
        filePath = filePath + FileConstants.FILE_DELIMITER + fileName.split("_")[0];
        File file = new File(filePath);
        if (!(file.exists() && file.isDirectory())) {
            log.info("创建的文件夹：{}", filePath);
            //创建文件夹
            boolean mkdirs = file.mkdirs();
            log.info("创建文件夹{}", mkdirs ? "成功" : "失败");
        }
        filePath = filePath + FileConstants.FILE_DELIMITER + fileName + FileConstants.JAVA_SUFFIX;
        File classFile = new File(filePath);
        if (!(classFile.exists() && classFile.isFile())) {
            try {
                log.info("创建的文件：{}", filePath);
                boolean newFile = classFile.createNewFile();
                log.info("创建文件{}", newFile ? "成功" : "失败");
            } catch (IOException e) {
                log.error("创建 [{}] 文件失败 --> {}", fileName, e.getMessage(), e);
                throw new RuntimeException(e);
            }
        }
        return classFile;
    }

    /**
     * 写文件
     */
    public static void writeFile(String filePath, String fileName, String content) {
        log.info("写文件 ---> 入参：[filePath:{},fileName:{},content{}]", filePath, fileName, content);
        File javaFile = createJavaFile(filePath, fileName);
        try {
            BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(javaFile));
            bufferedWriter.write(content);
            bufferedWriter.flush();
            bufferedWriter.close();
        } catch (IOException e) {
            log.error("保存[{}]文件失败 ---> {}", fileName, e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 读文件
     */
    public static String readFile(String filePath, String fileName) {
        try {
            File file = new File(filePath + FileConstants.FILE_DELIMITER + fileName.split("_")[0] + FileConstants.FILE_DELIMITER + fileName + FileConstants.JAVA_SUFFIX);
            BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
            String content = "";
            String len;
            while ((len = bufferedReader.readLine()) != null) {
                content += len + "\n";
            }
            return content;
        } catch (Exception e) {
            log.error("读取[{}]文件失败 ---> {}", fileName, e.getMessage(), e);
            return "";
        }
    }

    public static void main(String[] args) {

        String str = "package com.muyu.rule.test;\n" +
                "\n" +
                "import com.muyu.etl.rule.engine.bean.annotation.EngineParam;\n" +
                "import org.apache.logging.log4j.LogManager;\n" +
                "import org.apache.logging.log4j.Logger;\n" +
                "\n" +
                "/**\n" +
                " * @Author WangXin\n" +
                " * @Data 2024/8/23\n" +
                " * @Description\n" +
                " * @Version 1.0.0\n" +
                " */\n" +
                "public class XieYaRu {\n" +
                "\n" +
                "    private static final Logger log = LogManager.getLogger(XieYaRu.class.getName());\n" +
                "\n" +
                "    public String test(@EngineParam(name = \"phone\") String phone,@EngineParam(name = \"pat\") String pat){\n" +
                "        log.info(\"程序入参：phone -> {} ,pat -> {}\",phone,pat);\n" +
                "        if (phone.matches(pat)){\n" +
                "            return phone;\n" +
                "        }else {\n" +
                "            throw new RuntimeException(\"手机号格式错误\");\n" +
                "        }\n" +
                "\n" +
                "    }\n" +
                "}";

        writeFile("C:\\Users\\wx\\Desktop\\测试", "XieYaRu", str);

        String xieYaRu = readFile("C:\\Users\\wx\\Desktop\\测试", "XieYaRu");

        System.out.println(xieYaRu);
    }

}
