package com.muyu.etl.rule.service.impl;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.utils.OssUtil;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.utils.sign.Base64;
import com.muyu.etl.data.access.domain.DataAnalysis;
import com.muyu.etl.data.access.domain.req.EtlDataSqlReq;
import com.muyu.etl.data.access.domain.resp.TableInfoResp;
import com.muyu.etl.data.access.remote.RemoteDataAnalysisService;
import com.muyu.etl.data.access.remote.RemoteEtlDataService;
import com.muyu.etl.data.access.remote.RemoteTableInfoService;
import com.muyu.etl.domain.DataStructure;
import com.muyu.etl.rule.domain.RuleInfo;
import com.muyu.etl.rule.engine.bean.constant.FileConstants;
import com.muyu.etl.rule.engine.core.EngineContainer;
import com.muyu.etl.rule.engine.core.EngineExecution;
import com.muyu.etl.rule.engine.core.classLoad.JavaBinaryClassLoader;
import com.muyu.etl.rule.engine.core.complier.SourceCodeCompiler;
import com.muyu.etl.rule.engine.util.FileLoadUtil;
import com.muyu.etl.rule.basic.BasicEngine;
import com.muyu.etl.rule.config.EngineConfig;
import com.muyu.etl.rule.domain.RuleVersion;
import com.muyu.etl.rule.domain.req.DataSqlField;
import com.muyu.etl.rule.domain.req.DataTestReq;
import com.muyu.etl.rule.domain.req.ExecutionRuleVersionReq;
import com.muyu.etl.rule.domain.req.WriteFileReq;
import com.muyu.etl.rule.mapper.RuleVersionMapper;
import com.muyu.etl.rule.service.IRuleVersionService;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.codec.language.bm.RuleType;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import javax.annotation.Resource;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * 规则版本Service业务层处理
 *
 * @author wangXin
 * @date 2024-08-25
 */
@Service
@Log4j2
public class RuleVersionServiceImpl
        extends ServiceImpl<RuleVersionMapper, RuleVersion>
        implements IRuleVersionService {

    @Resource
    private EngineConfig engineConfig;
    @Resource
    private RemoteEtlDataService remoteEtlDataService;
    @Resource
    private RemoteTableInfoService remoteTableInfoService;
    @Resource
    private RemoteDataAnalysisService remoteDataAnalysisService;


    /**
     * 精确查询规则版本
     *
     * @param id 规则版本主键
     * @return 规则版本
     */
    @Override
    public RuleVersion selectRuleVersionById(Long id) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        Assert.notNull(id, "id不可为空");
        queryWrapper.eq(RuleVersion::getId, id);
        return this.getOne(queryWrapper);
    }


    /**
     * 查询规则版本列表
     *
     * @param ruleVersion 规则版本
     * @return 规则版本
     */
    @Override
    public List<RuleVersion> selectRuleVersionList(RuleVersion ruleVersion) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        if (StringUtils.isNotEmpty(ruleVersion.getRuleVersionName())) {
            queryWrapper.like(RuleVersion::getRuleVersionName, ruleVersion.getRuleVersionName());
        }
        if (StringUtils.isNotEmpty(ruleVersion.getRuleVersionCode())) {
            queryWrapper.eq(RuleVersion::getRuleVersionCode, ruleVersion.getRuleVersionCode());
        }
        if (ruleVersion.getRuleVersionIsActivation() != null) {
            queryWrapper.eq(RuleVersion::getRuleVersionIsActivation, ruleVersion.getRuleVersionIsActivation());
        }
        if (ruleVersion.getRuleVersionIsTest() != null) {
            queryWrapper.eq(RuleVersion::getRuleVersionIsTest, ruleVersion.getRuleVersionIsTest());
        }
        if (ruleVersion.getRuleVersionStatus() != null) {
            queryWrapper.eq(RuleVersion::getRuleVersionStatus, ruleVersion.getRuleVersionStatus());
        }
        return this.list(queryWrapper);
    }

    /**
     * 唯一 判断
     *
     * @param ruleVersion 规则版本
     * @return 规则版本
     */
    @Override
    public Boolean checkIdUnique(RuleVersion ruleVersion) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleVersion::getId, ruleVersion.getId());
        return this.count(queryWrapper) > 0;
    }

    /**
     * 通过规则id查询规则版本
     *
     * @param id 规则Id
     * @return 规则版本数据集合
     */
    @Override
    public List<RuleVersion> selectRuleVersionListByRuleId(Long id) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleVersion::getRuleId, id);
        return this.list(queryWrapper).stream()
                .peek(ruleVersion -> ruleVersion.setClassContent(readFile(ruleVersion.getRuleVersionClass())))
                .collect(Collectors.toList());
    }

    /**
     * 根据文件名称获取规则文件内容
     *
     * @param fileName 文件名称
     * @return 文件内容
     */
    @Override
    public String readFile(String fileName) {
        log.info("readFile请求入参：[fileName:{}]", fileName);
        return FileLoadUtil.readFile(engineConfig.getSourceLocation(), fileName);
    }

    /**
     * 根据文件名称写规则文件内容
     *
     * @param writeFileReq 文件写入请求对象
     */
    @Override
    public void writeFile(WriteFileReq writeFileReq) {
        String content = new String(Base64.decode(writeFileReq.getContent()));
        FileLoadUtil.writeFile(engineConfig.getSourceLocation(), writeFileReq.getFileName(), content);
        OssUtil.uploadFile(engineConfig.getSourceLocation() + FileConstants.FILE_DELIMITER + writeFileReq.getFileName().split("_")[0] + FileConstants.FILE_DELIMITER + writeFileReq.getFileName() + FileConstants.JAVA_SUFFIX);
    }

    /**
     * 执行规则
     *
     * @param executionRuleVersionReq 规则请求参数
     */
    @Override
    public Object executionRuleVersion(ExecutionRuleVersionReq executionRuleVersionReq) {
        return executionRule(executionRuleVersionReq);
    }

    /**
     * 执行值规则
     *
     * @param engineKey     规则类名
     * @param dataStructure 规则值处理数据
     */
    @Override
    public DataStructure executionRuleValueVersion(String engineKey, DataStructure dataStructure) {
        return executionValueRule(engineKey, dataStructure);
    }

    /**
     * 执行值规则
     *
     * @param engineKey        规则类名
     * @param dataStructureRow 规则行处理数据
     */
    @Override
    public DataStructure[][] executionRowRuleVersion(String engineKey, DataStructure[][] dataStructureRow) {
        ArrayList<DataStructure[]> dataStructures = new ArrayList<>();
        for (DataStructure[] dataStructure : dataStructureRow) {
            DataStructure[] dataStructure1 = executionValueRule(engineKey, dataStructure);
            if (dataStructure1 != null && dataStructure1.length > 0) {
                dataStructures.add(dataStructure1);
            }
        }
        return dataStructures.toArray(new DataStructure[dataStructures.size()][]);
    }

    /**
     * 执行值规则
     *
     * @param engineKey         规则类名
     * @param dataStructureRows 规则组处理数据
     */
    @Override
    public DataStructure[][] executionRowsRuleVersion(String engineKey, DataStructure[][] dataStructureRows) {
        return executionValueRule(engineKey, dataStructureRows);
    }

    /**
     * 远调查询数据源
     *
     * @return 数据库表二级联动
     */
    @Override
    public List<TableInfoResp> dataSourceList() {
        return remoteTableInfoService.findByTableName().getData();
    }

    @Override
    public List<DataAnalysis> dataFindById(Long id) {
        return remoteDataAnalysisService.findDataAnalysisByTableId(id).getData();
    }

    @Override
    public List<List<DataStructure>> testData(DataTestReq dataTestReq) {
        return remoteEtlDataService.selEtlData(EtlDataSqlReq.builder()
                .sql(spliceSql(dataTestReq))
                .basicId(dataTestReq.getBasicId())
                .build()).getData();
    }

    @Override
    public Boolean isClassExist(String className) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleVersion::getRuleVersionClass, className);
        return this.exists(queryWrapper);
    }



    /**
     * 凭借sql
     *
     * @param dataTestReq
     * @return
     */
    private String spliceSql(DataTestReq dataTestReq) {
        List<String> list = dataTestReq.getRows().stream().map(DataSqlField::getColumnName).collect(Collectors.toList());
        String sql = "SELECT " + list.stream().collect(Collectors.joining(",")) + " FROM " + dataTestReq.getTableName();
        log.info("拼接SQL语句为 --->[{}]", sql);
        return sql;
    }

    /**
     * 执行有参数的规则方法
     *
     * @param executionRuleVersionReq 规则请求参数
     * @return 返回结果集
     */
    private Object executionRule(ExecutionRuleVersionReq executionRuleVersionReq) {
        //将java文件编译成class文件
        SourceCodeCompiler.javaCompilerPath(engineConfig.getSourceLocation());
        //获取所有class文件的class对象
        Map<String, Class<?>> stringClassMap = JavaBinaryClassLoader
                .loadClassByLocation(engineConfig.getPackageName(),
                        engineConfig.sourceLocation);

        //实将所有class对象进行例化，并进行存储
        stringClassMap.forEach((k, v) -> {
            EngineContainer.loadEngineInstance(engineConfig.methodName, k, v);
        });
        return EngineExecution.argumentEngineExe(executionRuleVersionReq.getEngineKey(), executionRuleVersionReq.getParams());
    }

    /**
     * 将单文件编译为class文件
     */
    private String javaCompile(String fileName) {
        //将java文件编译成class文件
        SourceCodeCompiler.javaCompiler("", new File(getFilePath(fileName, FileConstants.JAVA_SUFFIX)));
        //返回oss中class文件地址
        return OssUtil.uploadFile(getFilePath(fileName, FileConstants.CLASS_SUFFIX));
    }

    /**
     * 将文件名转为特定文件地址
     *
     * @param fileName 文件名
     * @param endWih   文件后缀
     * @return 文件地址
     */
    private String getFilePath(String fileName, String endWih) {
        return engineConfig.getSourceLocation() + FileConstants.FILE_DELIMITER + fileName.split("_")[0] + FileConstants.FILE_DELIMITER + fileName + endWih;
    }


    /**
     * 执行无参的规则方法
     *
     * @param engineKey 规则类名
     */
    private <T> T executionValueRule(String engineKey, T t) {
        log.info("配置参数为 ------》 SourceLocation：{} | PackageName：{}", engineConfig.getSourceLocation(), engineConfig.getPackageName());
        log.info("请求类-----》 {}", engineKey);
        log.info("请求参数为----》{}", JSONObject.toJSONString(t));
        String path = engineConfig.getSourceLocation() + FileConstants.FILE_DELIMITER + engineKey.split("_")[0];
        //将java文件编译成class文件
        SourceCodeCompiler.javaCompiler(engineConfig.classConfigPath, path + "/classes", path, engineKey);
        //获取所有class文件的class对象
        Map<String, Class<?>> stringClassMap = JavaBinaryClassLoader
                .loadClassByLocation(engineConfig.getPackageName(),
                        path+ "/classes");
        log.info("[{}] 类编译成功 ", engineKey);
        //实将所有class对象进行例化，并进行存储
        stringClassMap.forEach((k, v) -> {
            EngineContainer.loadEngineInstance(engineConfig.methodName, k, v);
        });
        log.info("[{}] 类实例化成功", engineKey);
        ConcurrentHashMap<String, Object> instanceMap = EngineContainer.instanceMap;
        Object object = instanceMap.get(engineKey);
        log.info("返回值对象：object:{}", object);
        if (object != null && object instanceof BasicEngine<?>) {
            BasicEngine<T> engine = (BasicEngine<T>) object;
            engine.set(t);
            engine.execution();
            log.info("[{}]类加载成功", engineKey);
            T engineData = engine.get();
            log.info("[{}]处理数据结果为[{}]", engineKey, JSONObject.toJSONString(engineData));
            return engineData;
        } else {
            throw new RuntimeException("对象类型错误");
        }
    }


    public static void main(String[] args) {

//        String engineKey = "TelTuoMing";
//        String locationPath = "C:\\Users\\wx\\Desktop\\测试";
//        String page = "com.muyu.etl.rule";
//
//        SourceCodeCompiler.javaCompilerPath(locationPath);
//
//        Map<String, Class<?>> stringClassMap = JavaBinaryClassLoader
//                .loadClassByLocation(page, locationPath);
//
//        stringClassMap.forEach((k, v) -> {
//            EngineContainer.loadEngineInstance("run", k, v);
//        });
//
//        DataStructure dataStructure = new DataStructure();
//        dataStructure.setValue("15335893491");
//        dataStructure.setType(DataType.STRING);
//        dataStructure.setKey("tel");
//        dataStructure.setLabel("手机号");
//
//        ConcurrentHashMap<String, Object> instanceMap = EngineContainer.instanceMap;
//        Object object = instanceMap.get(engineKey);
//        if (object != null && object instanceof BasicEngine<?>) {
//            BasicEngine<DataStructure> engine = (BasicEngine<DataStructure>) object;
//            engine.set(dataStructure);
//            engine.execution();
//        }

    }
    /**
     * 查詢所有規則(字段,数据集,记录)规则下拉框
     * @return List<RuleInfo>
     */
    @Override
    public List<RuleInfo> getEtlRuleTypeByRuleId() {
        return List.of();
    }

    @Override
    public List<RuleVersion> getEtlRuleTypeVById(Long id) {
        return List.of();
    }


}
